%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab szkript az
%
% u_t = a*u_xx + b*u_x + c*u + f(x)
% u(t, 0) = p0
% u(t, L) = pL
% u(0, x) = u0
%
% PDE megoldására 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A kezdeti- és peremérték-feladat paraméterei
a = 1;
b = 10;
c = 0;
L = 1;
f = @(x) zeros(size(x));
p0 = 0;
pL = 0;
% Numerikus megoldás paraméterei
n = 30; % belső pontok száma + 1
tau = 0.0001; % időlépés nagysága
m = 1000; % időlépések száma
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Másodlagos paraméterek
h = L/n; % rácsállandó
X = 0:h:L; % rácspontok
X0 = X(2:end-1); % belső rácspontok
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Kezdeti feltétel megadása
u0 = [p0, sin(pi*X0/L).^20, pL];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Alkalmazott numerikus sémák
% Második derivált
% Centrális második derivált
D = 1/h^2*(diag((-2)*[0, ones(size(X0)), 0],0)+diag([ones(size(X0)), 0],-1)+diag([0, ones(size(X0))],1));
% Első derivált
% Előremutató derivált
A = 1/h*(diag((-1)*[0, ones(size(X0)), 0], 0)+diag([0, ones(size(X0))], 1));
% Hátranéző derivált
% A = 1/h*(diag(1*[0, ones(size(X0)), 0], 0)+(-1)*diag([ones(size(X0)), 0], -1));
% Centrális derivált
% A = 1/h*(diag(1/2*[0, ones(size(X0))], 1)+(-1)/2*diag([ones(size(X0)), 0], -1));
% Csonkolt egységmátrix
I = diag([0, ones(size(X0)), 0]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A mátrix és a forrástag előállítása
M = a*D + b*A + c*I;
g = [0, f(X0), 0];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Időléptetés és animáció
u_old = u0;
% Iteráció
for i = 1:m
% Új állapot meghatározása
    u_new = u_old + (tau * M*u_old')' + tau * g;
% Animáció
    plot(X, u_new, '-o');
% Tengelyek beállítása
    axis([0, L, 0, max(u0)]);
    drawnow;
% Állapot elmentése
    u_old = u_new;
end
